<?php
if ( !class_exists( 'Goldstar_Common' ) ) {

	class Goldstar_Common {

		public static $timeoutRequest = 1000;
		public static $_goldstar_dir_xml_name = 'goldstar-xml';
		private static $_goldstar_options;

		public static function request( $url ) {

			$request = new WP_Http;
			$result = $request->request( $url, array(
				'timeout' => self::$timeoutRequest,
			) );

			if ( is_wp_error( $result ) ) {
				return file_get_contents($url);
				throw new Exception($result->get_error_message());
			}

			return $result['body'];            
		}

		public static function createDirContentXMl( $goldstar_dir_xml_name ) {
			$_upload = WP_CONTENT_DIR . '/uploads';

			$_path_data_xml = $_upload;

			if ( !file_exists( $_path_data_xml ) ) {
				return array(
					'error' => true,
					'msg' => "Please create wp-content/uploads directory."
				);
			}

			if ( is_multisite() ) {
				$sites = get_blog_details(get_current_blog_id());
				$_final_path_data_xml = $_path_data_xml = $_upload.'/'.$goldstar_dir_xml_name.'/'.$sites->domain;
				if ( file_exists( $_path_data_xml ) ) {
					if ( !is_writable( $_path_data_xml ) ) {
						return array(
							'error' => true,
							'msg' => "Please set write permission to <strong>$_path_data_xml</strong>."
						);
					}
					else {
						return array(
							'error' => false,
							'msg' => '',
							'path' => $_final_path_data_xml,
						);
					}

				}
			}

			else {
				$_final_path_data_xml = $_path_data_xml = $_upload . '/' . $goldstar_dir_xml_name;
			}

			$_path_data_xml  = $_upload . '/' . $goldstar_dir_xml_name;

			if ( file_exists( $_path_data_xml ) ) {
				if ( !is_writable( $_path_data_xml ) ) {
					return array(
						'error' => true,
						'msg' => "Please set write permission to <strong>$_path_data_xml</strong>."
					);
				}
				else {
					if ( is_multisite() ) {
						mkdir( $_final_path_data_xml );
					}

					return array(
						'error' => false,
						'msg' => '',
						'path' => $_final_path_data_xml,
					);
				}
			}

			$_path_data_xml  = $_upload;

			if ( !is_writable( $_path_data_xml ) ) {
				return array(
					'error' => true,
					'msg' => "Please set write permission to <strong>$_path_data_xml</strong>."
				);
			}

			else {
				mkdir( $_upload . '/' . $goldstar_dir_xml_name );
				if ( is_multisite() ) {
					mkdir( $_final_path_data_xml );
				}

				return array(
					'error' => false,
					'msg' => '',
					'path' => $_final_path_data_xml,
				);
			}
		}

		public static function getCurrentTerritoryId() {
			$goldstar_options = get_option( 'goldstar_options' );

			return isset( $goldstar_options['territory_id'] ) ? intval( $goldstar_options['territory_id'] ) : 0;
		}

		/**
		 * Also depend on terriotry but get goldstar-0.xml for used
		 */
		public static function getXmlContentWithTerritory( $id = 0 ) {

			// Check error api first

			$goldstar_options = get_option( 'goldstar_options' );

			if ( $goldstar_options['api_valid'] === "0" || $goldstar_options['api_valid'] === "" ) {
				return array(
					'error' => true,
					'msg' => __('The API is invalid. Please input a valid API Key' . ' (<a href="' . admin_url('options-general.php?page=admin-goldstar') . '">Click here</a>)')
				);
			}

			// create directory if it does not exist yet
			$_arrReceived = Goldstar_Common::createDirContentXMl( Goldstar_Common::$_goldstar_dir_xml_name );

			if ( isset($_arrReceived['error']) && $_arrReceived['error'] === true ) {
				return $_arrReceived;
			}

			$_path_data_xml = $_arrReceived['path'];
			$filename = $_path_data_xml . '/' . 'goldstar-' . $id . '.xml';

			// Create file if it doesn't exist
			if ( ! file_exists( $filename ) ) {

				/* create file from xml */
				$url = Goldstar_API::$feed_link . '?api_key=' . $goldstar_options['api_key'];

				// set terrritory = 0
				$url .= "&territory_id=".$id;

				try {
					$xml = Goldstar_Common::request($url);
				}

				catch( Exception $e ) {
					return array(
						'error' => true,
						'msg' => $e->getMessage(),
					);
				}

				$xml = simplexml_load_string($xml);

				//Handler error
				if ( isset( $xml->error ) ) {
					return array(
						'error' => true,
						'msg' => (string)$xml->error,
					);
				}

				$xml->asXML( $filename );
				@chmod( $filename, 0755 );
			}

			return simplexml_load_file( $filename );
		}

		/**
		 * Query xml by xpath
		 *
		 * @param type $arr_filter
		 */
		public static function get_xpath_query( $arr_filter ) {

			$arr_condition = array();

			if ( !empty($arr_filter['from_date'] ) ) {
				$_date           = str_replace( Goldstar_Shortcode::$delimiter_date, '', $arr_filter['from_date'] );
				$arr_condition[] = "translate(upcoming_dates/event_date/date,'-', '') >= '$_date'";
			}

			if ( !empty($arr_filter['to_date'] ) ) {
				$_date           = str_replace( Goldstar_Shortcode::$delimiter_date, '', $arr_filter['to_date'] );
				$arr_condition[] = "translate(upcoming_dates/event_date/date,'-', '') <= '$_date'";
			}

			if ( !empty( $arr_filter['location'] ) ) {
				$arr_condition[] = 'venue/address/locality="' . $arr_filter['location'] . '"';
			}

			$category_list_where = '';
			if ( empty($arr_filter['category'] ) ) {
				//throw new Exception("This function used when category is not empty");
			}

			if ( !empty($arr_filter['category'] ) ) {
				if ( is_array($arr_filter['category'] ) ) {
					$arr_categories_list = array();

					foreach ( $arr_filter['category'] as $category ) {
						$arr_categories_list[] = 'category_list/category/name="' . $category . '"';
					}

					$arr_condition[] = '(' . implode(' or ', $arr_categories_list) . ')';
				}

				else {
					$arr_condition[] = 'category_list/category/name="' . $arr_filter['category'] . '"';
				}
			}

			if ( !empty( $arr_filter['price'] ) ) {
				/* Parse price */
				if ( $arr_filter['price'] === 'free' ) {
					$arr_condition[] = "(translate(substring-before(our_price_range,'-'),' ','') = 'COMP' or our_price_range = 'COMP')";
				}

				elseif ( strpos($arr_filter['price'], '<') !== false ) {
					$_price          = (int) substr($arr_filter['price'], strpos($arr_filter['price'], '<') + 1);
					$arr_condition[] = "(number(translate(substring-before(our_price_range,'-'),'$','')) < $_price or number(translate(substring-after(our_price_range,'-'),'$','')) < $_price or number(translate(our_price_range,'$','')) < $_price )";
				}

				elseif ( strpos($arr_filter['price'], '>') !== false ) {
					$_price          = (int) substr($arr_filter['price'], strpos($arr_filter['price'], '>') + 1);
					$arr_condition[] = "(number(translate(substring-before(our_price_range,'-'),'$','')) >= $_price or number(translate(substring-after(our_price_range,'-'),'$','')) >= $_price or number(translate(our_price_range,'$','')) >= $_price )";
				}

				elseif ( strpos($arr_filter['price'], '-') !== false ) {
					list($_start_price, $_end_price) = explode("-", $arr_filter['price']);
					$arr_condition[] = "(number(translate(substring-before(our_price_range,'-'),'$','')) >= $_start_price or number(translate(substring-after(our_price_range,'-'),'$','')) >= $_start_price or number(translate(our_price_range,'$','')) >= $_start_price ) ";
					$arr_condition[] = "(number(translate(substring-before(our_price_range,'-'),'$','')) < $_end_price or number(translate(substring-after(our_price_range,'-'),'$','')) < $_end_price or number(translate(our_price_range,'$','')) < $_end_price ) ";
				}
			}

			$xpath_query = 'event';

			if ( !empty($arr_condition ) ) {
				$xpath_query .= '[' . implode(" and ", $arr_condition) . ']';
			}

			return $xpath_query;
		}

		public static function getGoldstarOptions() {
			if ( self::$_goldstar_options !== null )
				return self::$_goldstar_options;

			return self::$_goldstar_options = get_option( 'goldstar_options' );
		}

	}

}